"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateParamCustomize = void 0;
require("reflect-metadata");
const index_1 = require("../../../../index");
const getDocDTO_1 = require("./getDocDTO");
const handleDataResponse = (classDTO, isArray = true) => {
    let target = {
        properties: {},
        required: ["data"]
    };
    const type = isArray ? "array" : "object";
    target.properties["data"] = {
        type,
        description: `The ${type} data ${classDTO.name}`,
    };
    const docDto = (0, getDocDTO_1.getDocDTO)(classDTO);
    // if type is object then assign properties
    if (type == "object") {
        target.properties["data"].properties = docDto.result;
        target.properties["data"].required = docDto.required;
    }
    else {
        target.properties["data"].items = {
            type: [String, Boolean, Number].includes(classDTO)
                ? classDTO.name.toLocaleLowerCase()
                : [Date].includes(classDTO)
                    ? 'string' : "object",
            properties: docDto.result,
            required: docDto.required
        };
    }
    return target;
};
const handleParamBody = (classDTO) => {
    return (0, index_1.getMetadataDTO)()
        .getFieldDTO(classDTO)
        .reduce((target, field) => {
        var _a, _b, _c, _d, _e, _f, _g, _h;
        let type = [String, Number, Boolean, Array].includes(field.type)
            ? field.type.name.toLocaleLowerCase()
            : [Date].includes(field.type) ? 'string' : "object";
        if (type === "object" && field.isEnum) {
            type = "string";
        }
        target.properties[field.name] = {
            type,
            description: field.description,
            enum: field.isEnum && type !== "array" ? Object.values((_a = field.vectorProps) === null || _a === void 0 ? void 0 : _a.elementDataType) : undefined
        };
        if (field.isRequire) {
            target.required.push(field.name);
        }
        // if type is object then assign properties
        if (type == "object") {
            const docDto = (0, getDocDTO_1.getDocDTO)(field.type);
            target.properties[field.name].properties = docDto.result;
            target.properties[field.name].required = docDto.required;
        }
        // if type is array then assign items
        if (type == "array") {
            if (((_b = field === null || field === void 0 ? void 0 : field.vectorProps) === null || _b === void 0 ? void 0 : _b.elementDataType) &&
                [index_1.DataType.BOOLEAN, index_1.DataType.NUMBER, index_1.DataType.STRING].includes((_c = field === null || field === void 0 ? void 0 : field.vectorProps) === null || _c === void 0 ? void 0 : _c.elementDataType)) {
                target.properties[field.name].items = {
                    type: field.isEnum ? "string" : ((_d = field === null || field === void 0 ? void 0 : field.vectorProps) === null || _d === void 0 ? void 0 : _d.elementDataType).toLocaleLowerCase(),
                    enum: field.isEnum ? Object.values((_e = field.vectorProps) === null || _e === void 0 ? void 0 : _e.elementDataType) : undefined
                };
            }
            else {
                const docDTO = (0, getDocDTO_1.getDocDTO)((_f = field === null || field === void 0 ? void 0 : field.vectorProps) === null || _f === void 0 ? void 0 : _f.elementDataType);
                target.properties[field.name].items = {
                    type: ((_g = field === null || field === void 0 ? void 0 : field.vectorProps) === null || _g === void 0 ? void 0 : _g.elementDataType) && !field.isEnum ? "object" : "string",
                    properties: docDTO.result,
                    required: docDTO.required,
                    enum: field.isEnum ? Object.values((_h = field.vectorProps) === null || _h === void 0 ? void 0 : _h.elementDataType) : undefined
                };
            }
        }
        return target;
    }, { properties: {}, required: [] });
};
const handleParamQuery = (classDTO) => {
    return (0, index_1.getMetadataDTO)()
        .getFieldDTO(classDTO)
        .map((field) => {
        var _a, _b, _c, _d, _e;
        const type = [String, Number, Boolean].includes(field.type)
            ? field.type.name.toLocaleLowerCase()
            : [Date].includes(field.type) ? 'string' :
                [Array].includes(field.type)
                    ? "array"
                    : "object";
        let data = {
            name: field.name,
            required: field.isRequire,
            description: field.description,
            schema: {
                type: type == "object" && field.isEnum ? "string" : type,
                enum: field.isEnum ? Object.values((_a = field.vectorProps) === null || _a === void 0 ? void 0 : _a.elementDataType) : undefined
            },
        };
        let dataObject = {};
        // if type is object then assign properties
        if (type == "object" && !field.isEnum) {
            const docDTO = (0, getDocDTO_1.getDocDTO)(field.type);
            dataObject = {
                content: {
                    "application/json": {
                        type: type,
                        schema: {
                            properties: docDTO.result,
                            required: docDTO.required
                        },
                    },
                },
            };
        }
        // if type is array then assign items
        if (type == "array") {
            let dataArray;
            if (field.isEnum || (((_b = field === null || field === void 0 ? void 0 : field.vectorProps) === null || _b === void 0 ? void 0 : _b.elementDataType) &&
                [index_1.DataType.BOOLEAN, index_1.DataType.NUMBER, index_1.DataType.STRING].includes((_c = field === null || field === void 0 ? void 0 : field.vectorProps) === null || _c === void 0 ? void 0 : _c.elementDataType))) {
                dataArray = {
                    type: field.isEnum ? "string" : ((_d = field === null || field === void 0 ? void 0 : field.vectorProps) === null || _d === void 0 ? void 0 : _d.elementDataType).toLocaleLowerCase(),
                    enum: field.isEnum ? Object.values((_e = field.vectorProps) === null || _e === void 0 ? void 0 : _e.elementDataType) : undefined
                };
            }
            else {
                // query don't have object
            }
            dataObject = {
                content: {
                    "application/json": {
                        type: type,
                        schema: {
                            items: dataArray,
                        },
                    },
                },
            };
        }
        return Object.assign(Object.assign({}, data), dataObject);
    });
};
const generateParamCustomize = (controller, handlerName) => {
    const paramConfig = Reflect.getMetadata(index_1.MetadataKeys.ROUTE_ARGS_METADATA, controller.constructorFn, handlerName);
    const paramTypes = Reflect.getMetadata(index_1.MetadataKeys.PARAM_TYPE, controller.instance, handlerName);
    const responseType = Reflect.getMetadata(index_1.MetadataKeys.RESPONSE_TYPE_METADATA, controller.instance, handlerName);
    const description = Reflect.getMetadata(index_1.MetadataKeys.API_DESCRIPTION_METADATA, controller.instance, handlerName);
    const requestDoc = {
        requestBody: {
            properties: {},
            required: []
        },
        parameters: [],
        requestResponse: !responseType
            ? {}
            : handleDataResponse(responseType.classResponse, responseType.isArray).properties,
    };
    paramConfig === null || paramConfig === void 0 ? void 0 : paramConfig.forEach((e) => {
        switch (e.type) {
            case index_1.RouteParamtypes.BODY:
                const dataBody = handleParamBody(paramTypes[e.index]);
                requestDoc.requestBody = {
                    properties: Object.assign(Object.assign({}, requestDoc.requestBody.properties), dataBody.properties),
                    required: [
                        ...dataBody.required,
                        ...requestDoc.requestBody.required
                    ]
                };
                break;
            case index_1.RouteParamtypes.PARAMS:
            case index_1.RouteParamtypes.QUERY:
                const fields = handleParamQuery(paramTypes[e.index]);
                requestDoc.parameters = requestDoc.parameters.concat(fields.map((field) => (Object.assign(Object.assign({}, field), { in: e.type.toLocaleLowerCase() }))));
                break;
        }
    });
    return {
        description,
        requestBody: {
            content: {
                "application/json": {
                    schema: {
                        type: "object",
                        properties: requestDoc.requestBody.properties,
                        required: requestDoc.requestBody.required
                    },
                },
            },
        },
        parameters: requestDoc.parameters,
        requestResponse: {
            "200": {
                description: "Successfully",
                content: {
                    "application/json": {
                        schema: {
                            type: "object",
                            properties: requestDoc.requestResponse,
                        },
                    },
                },
            },
        },
    };
};
exports.generateParamCustomize = generateParamCustomize;
