"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CRUDController = void 0;
const queryBuilder_1 = require("../queryBuilder");
const constant_1 = require("../global/constant");
const responseBuilder_1 = require("../responseBuilder");
const pickData_1 = require("../helpers/pickData");
const consts_1 = require("../authorization/consts");
const helpers_1 = require("../helpers");
const errorBuilder_1 = require("../errorBuilder");
const relationship_1 = require("../decorators/interfaces/relationship");
const oDataTransform_1 = require("../helpers/oDataTransform");
const AbstractHttpController_1 = require("./AbstractHttpController");
/*====================+
 |CLASS CRUDCONTROLLER|
 |~-~-~-~QUY NT~-~-~-~|
 +====================*/
class CRUDController extends AbstractHttpController_1.AbstractHttpController {
    constructor(queryBuilder) {
        super();
        this.queryBuilder = queryBuilder;
    }
    getQueries(req, queriesString) {
        const queries = new queryBuilder_1.QueryRequest(this.queryBuilder).getQueries(queriesString);
        // check current user
        if (req.user && req.user.roleName !== consts_1.RoleAdmin) {
            const columnHasCurrentUser = this.queryBuilder.getColumns().find(e => e.isCurrentUser);
            if (columnHasCurrentUser) {
                if (queries[columnHasCurrentUser.name] && queries[columnHasCurrentUser.name] != req.user.id) {
                    throw errorBuilder_1.ErrorBuilder.accessDenied();
                }
                queries[columnHasCurrentUser.name] = {
                    $eq: (0, helpers_1.objectId)(req.user.id)
                };
            }
        }
        return queries;
    }
    getPagination(req) {
        var _a;
        const paginate = (_a = req.query) === null || _a === void 0 ? void 0 : _a.paginate;
        const paginateParse = paginate
            ? (0, pickData_1.pickData)(JSON.parse(paginate), ["page", "limit"])
            : constant_1.REQUEST_PAGINATE;
        return {
            page: (paginateParse === null || paginateParse === void 0 ? void 0 : paginateParse.page) || constant_1.REQUEST_PAGINATE.page,
            limit: (paginateParse === null || paginateParse === void 0 ? void 0 : paginateParse.limit) || constant_1.REQUEST_PAGINATE.limit,
        };
    }
    getWith(req, _with, queryBuilder = this.queryBuilder, deep = 1) {
        if (deep > 3) {
            throw errorBuilder_1.ErrorBuilder.validator('The deep of with must less than or equal to 3');
        }
        if (!Array.isArray(_with)) {
            throw errorBuilder_1.ErrorBuilder.validator('The with must be an array');
        }
        return _with.map((value) => {
            if (!value.model) {
                throw errorBuilder_1.ErrorBuilder.validator('The model is required in with.');
            }
            Object.keys(value).forEach(key => {
                if (!['model', 'filter', 'with', 'alias'].includes(key)) {
                    throw errorBuilder_1.ErrorBuilder.validator(`The ${key} not allow.`);
                }
            });
            const relation = queryBuilder.getRelations().find((relation) => relation.fieldName == value.model);
            if (!relation) {
                throw errorBuilder_1.ErrorBuilder.validator(`The relationship ${value.model} is not found`);
            }
            const model = relation.type === relationship_1.TypeRelation.BELONG_TO ? relation.refs.modelParent() : relation.refs.modelChild();
            if (value.filter) {
                const queries = new queryBuilder_1.QueryRequest((0, helpers_1.getQueryModel)(model)).getQueries(JSON.stringify(value.filter));
                // check current user
                if (req.user && req.user.roleName !== consts_1.RoleAdmin) {
                    const columnHasCurrentUser = this.queryBuilder.getColumns().find(e => e.isCurrentUser);
                    if (columnHasCurrentUser) {
                        if (queries[columnHasCurrentUser.name] && queries[columnHasCurrentUser.name] != req.user.id) {
                            throw errorBuilder_1.ErrorBuilder.accessDenied();
                        }
                        queries[columnHasCurrentUser.name] = {
                            $eq: (0, helpers_1.objectId)(req.user.id)
                        };
                    }
                }
                value.filter = queries;
            }
            if (value.with) {
                value.with = this.getWith(req, value.with, (0, helpers_1.getQueryModel)(model), deep + 1);
            }
            return value;
        });
    }
    getOperatorFilter(req, operatorFilter, queryBuilder = this.queryBuilder) {
        try {
            const expression = (0, oDataTransform_1.parseExpression)(operatorFilter);
            let queries = (0, oDataTransform_1.transform)(expression.type, expression.left, expression.right);
            if (req.user && req.user.roleName !== consts_1.RoleAdmin) {
                const columnHasCurrentUser = queryBuilder.getColumns().find(e => e.isCurrentUser);
                if (columnHasCurrentUser) {
                    queries = {
                        $and: [
                            {
                                [columnHasCurrentUser.name]: {
                                    $eq: (0, helpers_1.objectId)(req.user.id)
                                }
                            },
                            Object.assign({}, queries)
                        ]
                    };
                }
            }
            queries = new queryBuilder_1.QueryRequest(queryBuilder).validateAndConverOperatorFilter(queries);
            return queries;
        }
        catch (error) {
            const message = error instanceof errorBuilder_1.ErrorBuilder ? error.message : "The $filter is invalid";
            throw errorBuilder_1.ErrorBuilder.validator(message);
        }
    }
    getOperatorWith(req, _with, queryBuilder = this.queryBuilder, deep = 1) {
        try {
            if (deep > 3) {
                throw errorBuilder_1.ErrorBuilder.validator('The deep of with must less than or equal to 3');
            }
            if (!Array.isArray(_with)) {
                throw errorBuilder_1.ErrorBuilder.validator('The with must be an array');
            }
            return _with.map((e) => {
                if (!e.model) {
                    throw errorBuilder_1.ErrorBuilder.validator('The model is required in with.');
                }
                const value = {
                    model: e.model,
                };
                const relation = queryBuilder.getRelations().find((relation) => relation.fieldName == e.model);
                if (!relation) {
                    throw errorBuilder_1.ErrorBuilder.validator(`The relationship ${e.model} is not found`);
                }
                const model = relation.type === relationship_1.TypeRelation.BELONG_TO ? relation.refs.modelParent() : relation.refs.modelChild();
                if (e.filter) {
                    value.filter = this.getOperatorFilter(req, e.filter, (0, helpers_1.getQueryModel)(model));
                }
                if (e.with) {
                    value.with = this.getOperatorWith(req, e.with, (0, helpers_1.getQueryModel)(model), deep + 1);
                }
                return value;
            });
        }
        catch (error) {
            const message = error instanceof errorBuilder_1.ErrorBuilder ? error.message : "The $with is invalid";
            throw errorBuilder_1.ErrorBuilder.validator(message);
        }
    }
    get(req) {
        return __awaiter(this, void 0, void 0, function* () {
            const queries = req.query.$filter ? this.getOperatorFilter(req, req.query.$filter) : this.getQueries(req, req.query.queries);
            const paginate = this.getPagination(req);
            if (req.query.$with) {
                const _with = this.getOperatorWith(req, (0, oDataTransform_1.parseWith)(req.query.$with));
                this.queryBuilder.with(_with);
            }
            else if (req.query.with) {
                const _with = this.getWith(req, JSON.parse(req.query.with));
                this.queryBuilder.with(_with);
            }
            return this.queryBuilder.paginate(queries, paginate, { fromCRUD: true });
        });
    }
    create(req) {
        return __awaiter(this, void 0, void 0, function* () {
            // check current user
            if (req.user) {
                const columnHasCurrentUser = this.queryBuilder.getColumns().find(e => e.isCurrentUser);
                if (columnHasCurrentUser) {
                    if (req.body[columnHasCurrentUser.name]) {
                        if (req.body[columnHasCurrentUser.name] != req.user.id && req.user.roleName !== consts_1.RoleAdmin) {
                            throw errorBuilder_1.ErrorBuilder.accessDenied();
                        }
                    }
                    else {
                        req.body[columnHasCurrentUser.name] = req.user.id;
                    }
                }
            }
            const data = yield this.queryBuilder.createOne(req.body);
            return { data };
        });
    }
    update(req) {
        return __awaiter(this, void 0, void 0, function* () {
            const queries = req.query.$filter ? this.getOperatorFilter(req, req.query.$filter) : this.getQueries(req, req.query.queries);
            const dataUpdate = (0, pickData_1.pickData)(req.body, this.queryBuilder.getNameColumns());
            // check current user
            if (req.user && req.user.roleName !== consts_1.RoleAdmin) {
                const columnHasCurrentUser = this.queryBuilder.getColumns().find(e => e.isCurrentUser);
                if (columnHasCurrentUser) {
                    if (dataUpdate[columnHasCurrentUser.name] && dataUpdate[columnHasCurrentUser.name] != req.user.id) {
                        throw errorBuilder_1.ErrorBuilder.accessDenied();
                    }
                    queries[columnHasCurrentUser.name] = {
                        $eq: (0, helpers_1.objectId)(req.user.id)
                    };
                }
            }
            const data = yield this.queryBuilder.updateMany(queries, dataUpdate);
            return { data };
        });
    }
    delete(req) {
        return __awaiter(this, void 0, void 0, function* () {
            const queries = req.query.$filter ? this.getOperatorFilter(req, req.query.$filter) : this.getQueries(req, req.query.queries);
            const data = yield this.queryBuilder.delete(queries);
            return responseBuilder_1.ResponseBuilder.json(data);
        });
    }
}
exports.CRUDController = CRUDController;
