"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.injector = exports.Injector = void 0;
const constants_1 = require("../metadata/constants");
class Injector extends Map {
    constructor() {
        super(...arguments);
        this.containers = [];
    }
    resolve(target, isNew = false, alias) {
        const tokens = Reflect.getMetadata(constants_1.MetadataKeys.PARAM_TYPE, target) || [];
        const injections = tokens.map((token) => this.resolve(token));
        const classInstance = this.get(target);
        if (classInstance && !isNew) {
            return classInstance;
        }
        const newClassInstance = new target(...injections);
        this.set(target, newClassInstance);
        if (alias) {
            this.set(alias, newClassInstance);
        }
        if (!this.containers.includes(target))
            this.containers.push(target);
        return newClassInstance;
    }
    release() {
        for (const value of this.values()) {
            if (typeof value["release"] === "function") {
                value["release"]();
            }
        }
        this.clear();
    }
}
exports.Injector = Injector;
exports.injector = new Injector();
