"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MetadataTable = void 0;
const mongodb_1 = require("mongodb");
const commons_1 = require("../helpers/commons");
class MetadataTable {
    constructor() {
        this.data = new Map();
    }
    getTables() {
        return this.data;
    }
    getColumn(tableClass) {
        const tables = this.data.get(tableClass);
        if (!tables) {
            throw Error(`${tableClass.name} not found`);
        }
        return tables.columns;
    }
    getRelation(tableClass) {
        const table = this.data.get(tableClass);
        if (!table) {
            throw Error(`${tableClass.name} not found`);
        }
        return table.relations;
    }
    pushColumn(tableClass, column) {
        const table = this.data.get(tableClass);
        if (table) {
            if (!table.columns.find((e) => e.name == column.name)) {
                table.columns.push(column);
                this.data.set(tableClass, table);
            }
        }
        else
            throw Error(`${tableClass.name} not found`);
    }
    pushRelation(tableClass, relation) {
        const table = this.data.get(tableClass);
        if (table) {
            if (!table.relations.find((e) => e.fieldName == relation.fieldName)) {
                table.relations.push(relation);
                this.data.set(tableClass, table);
            }
        }
        else
            throw Error(`${tableClass.name} not found`);
    }
    getModelTable(tableClass) {
        const tableName = (0, commons_1.convertToPlural)(tableClass.name);
        if (!this.data.get(tableClass)) {
            this.data.set(tableClass, {
                name: tableName,
                columns: [
                    {
                        name: "_id",
                        type: mongodb_1.ObjectId,
                        isPrimaryKey: true,
                        description: `The id of ${tableName}`
                    },
                    {
                        name: "createdAt",
                        type: Number,
                        description: `The date created of ${tableName}`
                    },
                    {
                        name: "updatedAt",
                        type: Number,
                        description: `The date updated of ${tableName}`
                    },
                ],
                relations: []
            });
        }
        return tableName;
    }
    getAllTableName() {
        let tableNames = [];
        this.data.forEach((item) => {
            tableNames.push(item.name);
        });
        return tableNames;
    }
}
exports.MetadataTable = MetadataTable;
