import { Collection, ObjectId } from "mongodb";
import { IListCollection } from "../connections/mongodb";
import { QueryCondition, FindOptions } from "./interfaces/QueryCondition";
import { IColumn } from "../decorators/interfaces/table";
import { IPagination } from "../global/interface";
import { WithRelation } from "./interfaces/WithRelation";
export declare abstract class QueryBuilder<T> {
    protected collection?: Collection;
    protected withRelations: WithRelation<T, any>[];
    protected getTableName(): keyof IListCollection;
    getColumns(): IColumn[];
    getNameColumns(): string[];
    getRelations(): import("../decorators/interfaces/relationship").IRelationship<any, any>[];
    private getCollection;
    with<K>(params: WithRelation<T, K> | WithRelation<T, K>[]): this;
    findById(_id: string | ObjectId): Promise<T | null>;
    findOne(filter: QueryCondition<T>): Promise<T | null>;
    find(filter: QueryCondition<T>, options?: FindOptions<T>): Promise<T[]>;
    paginate(filter: QueryCondition<T>, paginate: IPagination, options?: FindOptions<T>): Promise<{
        data: T[];
        totalCount: number;
    }>;
    count(filter: QueryCondition<T>): Promise<number>;
    createOne(item: Partial<T>): Promise<T>;
    updateMany(filter: QueryCondition<T>, item: Partial<T>): Promise<T[]>;
    delete(filter: QueryCondition<T>): Promise<import("mongodb").DeleteResult>;
    deleteMany(filter: QueryCondition<T>): Promise<import("mongodb").DeleteResult>;
    private validateParamBeforeSaveDB;
    mappingDataRelation(data: T[], options?: FindOptions<T>): Promise<T[]>;
    prepareDataRelation(data: T[], options?: FindOptions<T>): Promise<any[]>;
}
export declare class QueryRequest<T> {
    readonly queryBuilder: QueryBuilder<T>;
    constructor(queryBuilder: QueryBuilder<T>);
    private convertQueriesIncludeObjectId;
    private convertQueriesIncludeString;
    private convertQueriesIncludeNumber;
    getQueries(queries: string | null): QueryCondition<T>;
    validateAndConverOperatorFilter(queries: QueryCondition<T>): QueryCondition<T>;
}
