type Op = "$eq" | "$ne" | "$lt" | "$lte" | "$gt" | "$gte";
type OpArray = "$in" | "$nin";
type MultipleOp = "$and" | "$or";
export type SingleOperator<T> = {
    [key in keyof T]?: {
        [op in Op]?: T[key];
    };
};
export type SingleArrayOperator<T> = {
    [key in keyof T]?: {
        [op in OpArray]?: T[key][];
    };
};
export type SpecialOperfator<T> = {
    [key in keyof T]?: {
        $like?: T[key] extends string ? string : never;
        $regex?: T[key] extends string ? RegExp | string : never;
        $exists?: boolean;
        $options?: "i";
        $eq?: T[key];
        $lt?: T[key];
        $lte?: T[key];
        $gt?: T[key];
        $gte?: T[key];
    };
};
export type MultipleOperator<T> = {
    [key in MultipleOp]?: (SingleOperator<T> | SingleArrayOperator<T> | SpecialOperfator<T> | MultipleOperator<T>)[];
};
export type SingleQueryCondition<T> = SingleOperator<T> | SingleArrayOperator<T> | SpecialOperfator<T>;
export type QueryCondition<T> = SingleQueryCondition<T> | MultipleOperator<T>;
export type Sort = string | Exclude<SortDirection, {
    $meta: string;
}> | string[] | {
    [key: string]: SortDirection;
} | Map<string, SortDirection> | [string, SortDirection][] | [string, SortDirection];
export type SortDirection = 1 | -1 | 'asc' | 'desc' | 'ascending' | 'descending' | {
    $meta: string;
};
export type FindOptions<T> = {
    /** Sets the limit of documents returned in the query. */
    limit?: number;
    /** Set to sort the documents coming back from the query. Array of indexes, `[['a', 1]]` etc. */
    sort?: Sort;
    /** The fields to return in the query. Object of fields to either include or exclude (one of, not both), `{'a':1, 'b': 1}` **or** `{'a': 0, 'b': 0}` */
    projection?: {
        [key in keyof T]?: 0 | 1;
    };
    /** Set to skip N documents ahead in your query (useful for pagination). */
    skip?: number;
    fromCRUD?: boolean;
};
export {};
